<?php

class Controller_Api_V0_StudyPlan extends Controller_Handler_Api {
    private function normalizeDisciplinesData($disciplineData, $facultyID) {
        if (!isset($disciplineData->subjectID) && isset($disciplineData->externalID)) {
            $disciplineData->subjectID = Model_Subject::withExternalID($disciplineData->externalID, $disciplineData->name, '', $facultyID);
        }

        foreach ($disciplineData->teachers as $teacherData) {
            if (empty($teacherData->hashSnils)) {
                Log::instance()->add(Log::WARNING, '{0} {1}: {2}', array(
                    '{0}' => 'SYNC_DISCIPLINES',
                    '{1}' => 'Discipline with empty teacher\'s hash INILA',
                    '{2}' => json_encode($disciplineData),
                ));
            } else {
                $ID = Model_Teacher::withINILA($teacherData->hashSnils);
                if ($ID != -1) {
                    $disciplineData->teacherIDs[] = $ID;
                }
            }
        }

        if (empty($disciplineData->teacherIDs)) {
            if (!empty($disciplineData->teachers)) {
                Log::instance()->add(Log::WARNING, '{0} {1}: {2}', array(
                    '{0}' => 'SYNC_DISCIPLINES',
                    '{1}' => 'Discipline without teachers',
                    '{2}' => json_encode($disciplineData),
                ));
            }
        }

        switch ($disciplineData->type) {
            case 'Экзамен':
                $disciplineData->type = Model_Discipline::EXAM;
                break;

            case 'Зачет':
                $disciplineData->type = Model_Discipline::CREDIT;
                break;

            default:
                throw new InvalidArgumentException('Discipline has bad type');
        }

        return $disciplineData;
    }

    private function processDiscipline($disciplineData, $semesterID, $facultyID) {
        if (isset($disciplineData->subjectID)) {
            $discipline = Model_Discipline::find($facultyID, $semesterID, $disciplineData->subjectID, $disciplineData->type);
        }

        $lTeacherIDs = [];
        if (isset($discipline)) {
            if (isset($disciplineData->gradeID)) {
                $discipline->changeGradeUnsafe($disciplineData->gradeID);
            }

            // Сравнить список пришедших преподавателей и тех, которые уже прикреплены
            if (!empty($disciplineData->teacherIDs)) {
                $lTeacherIDs = array_diff($disciplineData->teacherIDs, $discipline->getTeachers()->as_array());
            }
        } else {
            if (empty($disciplineData->teacherIDs)) {
                throw new InvalidArgumentException('Cannot create discipline without teachers');
            }

            $discipline = Model_Discipline::make()
                ->author($disciplineData->teacherIDs[0])
                ->subject($disciplineData->subjectID)
                ->semester($semesterID)
                ->type($disciplineData->type)
                ->faculty($facultyID)
                ->create();

            $lTeacher = $discipline->getTeachers()->as_array();
            foreach ($lTeacher as $teacher) {
                $lTeacherIDs[] = $teacher['ID'];
            }
        }

        foreach ($lTeacherIDs as $teacherID) {
            $teacher = Model_Teacher::with($teacherID);
            $discipline->bind($teacher);
        }

        return (int)$discipline->ID;
    }

    private function processStudyPlan($studyPlanData, $year, $semesterID, $facultyID) {
        if (!isset($studyPlanData->id) && isset($studyPlanData->externalID)) {
            $studyPlan = Model_Plan::withExternalID($studyPlanData->externalID);
            if (!isset($studyPlan)) {
                $studyPlan = Model_Plan::make()
                    ->externalID($studyPlanData->externalID)
                    ->year($year)
                    ->faculty($facultyID)
                    ->create();
            } else {
                $studyPlan->changeInfo($year, $facultyID);
            }
        } elseif (isset($studyPlanData->id)) {
            $studyPlan = Model_Plan::load($studyPlanData->id);
        } else {
            throw new InvalidArgumentException('Study plan has no id or external id!');
        }

        $disciplineIDs = [];
        foreach ($studyPlanData->disciplines as &$disciplineData) {
            try {
                $disciplineData = $this->normalizeDisciplinesData($disciplineData, $facultyID);
                $disciplineID = $this->processDiscipline($disciplineData, $semesterID, $facultyID);
                $studyPlan->bindDiscipline($disciplineID);
                $disciplineIDs[] = $disciplineID;
            } catch (Exception $e) {
                Log::instance()->add(Log::WARNING, '{0} {1}: {2}', array(
                    '{0}' => 'SYNC_DISCIPLINES',
                    '{1}' => $e->getMessage(),
                    '{2}' => json_encode($disciplineData),
                ));
            }
        }

        return $disciplineIDs;
    }

//    /**
//     * @api {put} api/v0/studyPlan Add new study plan(s)
//     * @apiName Add new study plan
//     * @apiGroup Study plans
//     * @apiVersion 0.1.2
//     * @apiParam {String} token Api key
//     * @apiParam {String} [batch] Flag about massive addition
//     * @apiParam {String} externalId
//     * @apiParam {Array} disciplines
//     */
    public function action_put_index() {
        $res = [];

        try {
            if ($this->request->query('batch') !== NULL) {
                $data = json_decode($this->request->body());

                $year = (int)$data->year;
                $semesterID = (int)Model_Semesters::find($year, (int)$data->semester)->ID;
                $facultyID = Model_Faculties::getIdByExternalID($data->faculty);

                foreach ($data->plans as $studyPlanData) {
                    $res[] = $this->processStudyPlan($studyPlanData, $year, $semesterID, $facultyID);
                }
            } else {
                throw new InvalidArgumentException('No batch data received!');
            }
        } catch (Exception $e) {
            $this->badRequestError($e->getMessage());
        }

        return $res;
    }
}
